<?php

/**
 * Install hook. Check if the plugin uses the 0.8.0 data model.
 */
function nktagcloud_install() {
	if ( !get_option('nktagcloud') ) {
		nktagcloud_migrate_to_0_8_0();
	}
}

/**
 *
 * Reset the tag cloud to default values
 *
 * @since 0.8.0
 */
function nktagcloud_reset() {
	$default = array(
		'version'	=> '0.8.0',
		'config'	=> array(
			'categories'			=> 'No',		// Include catsegories
			'decor'					=> 'notouch',	// CSS for tag links
			'exclude'				=> '',
			'format'				=> 'list',
			'homelink'				=> 'No',		// Remove link to homepage
			'inject_count'			=> 'No',		// Add tag counter
			'inject_count_outside'	=> 'Yes',		// Do not link the counter
			'include'				=> '',
			'largest'				=> '22',
			'number'				=> '0',			// Tag limit
			'order'					=> 'ASC',
			'orderby'				=> 'name',
			'rmcss'					=> 'No',		// Remove plugin CSS
			'replace'				=> 'No',		// blanks to &nbsp;
			'smallest'				=> '8',
			'title'					=> 'Tags',
			'unit'					=> 'pt',
		),
	);
	update_option( 'nktagcloud', $default );
}

/**
 *
 * Migrate from earlier messy option model to cleaner layout
 *
 * @since 0.8.0
 */
function nktagcloud_migrate_to_0_8_0() {
	// It's save to reset since the new option doesn't exist pre 0.8.0
	nktagcloud_reset();
	$option = get_option( 'nktagcloud' );
	// All pre 0.8.0 settings
	$settings = array( 'categories', 'decor', 'exclude', 'format', 'homelink', 'include', 'inject_count', 'inject_count_outside', 'largest', 'number', 'order', 'orderby', 'replace', 'rmcss', 'smallest', 'title', 'unit' );
	foreach ($settings as $setting) {
		// Import and delete all old settings
		$option['config'][$setting] = get_option( "nktagcloud_$setting" );
		delete_option( "nktagcloud_$setting" );
	}
	update_option( 'nktagcloud', $option );
}

/**
 * The widget controls
 *
 * TODO maybe add basic configuration here?
 */
function nktagcloud_control() { ?>
	There were too many options, so I had to move the configuration
	to a <a href="<?php bloginfo('url') ?>/wp-admin/options-general.php?page=nktagcloud">settings page</a>. <?php
}


/**
 * Add menu configuration page to the menu
 */
function nktagcloud_add_pages() {
    add_options_page('Better Tag Cloud', 'Better Tag Cloud', 10, 'nktagcloud', 'nktagcloud_options_page');
}

/**
 * The configuration page
 *
 * TODO this is messy...
 */
function nktagcloud_options_page() {
	#echo '<pre>'; var_dump( $_POST ); echo '</pre>'; echo '<hr>';
	if ( $_POST['config'] ) {
		function_exists( 'check_admin_referer ') ? check_admin_referer( 'nktagcloud' ) : null;
		$option = get_option( 'nktagcloud' );
		$option['config'] = $_POST['config'];
		update_option( 'nktagcloud', $option );
		$config = $option['config'];
	}
	elseif ( $_POST['reset'] ) {
		function_exists( 'check_admin_referer ') ? check_admin_referer( 'nktagcloud' ) : null;
		nktagcloud_reset();
	}
	$option = get_option( 'nktagcloud' );
	$config = $option['config']; ?>

	<div class="wrap" style="margin: 2mm; max-width: 100ex;">
		<h2>Better Tag Cloud</h2>
		<p> 
			The default tag cloud widget that comes with wordpress is very simple. This widget makes the options for wp_tag_cloud() available to this new tag cloud widget. It has new features as well.
			<br />
			Visit the <a href="http://www.nkuttler.de/wordpress/nktagcloud/">plugin's home page</a> to leave comments or <a href="http://www.wordpress.org/extend/plugins/nktagcloud/">rate this widget</a>.
		</p>
		
		<h2>Widget Configuration</h2>
		<p>Please see <a href="http://codex.wordpress.org/Template_Tags/wp_tag_cloud" target="_blank">the wp_tag_cloud() page</a> for a detailed description of the options.</p>
		<form action="" method="post">
			<?php function_exists('wp_nonce_field') ? wp_nonce_field('nktagcloud') : null; ?>
			<?php nktagcloud_input( 'Title', 'title', 15, $config['title'] ) ?> 
			<br />
			<?php nktagcloud_input( 'Smallest font size', 'smallest', 4, $config['smallest'] ) ?> 
			<?php nktagcloud_input( 'Largest font size', 'largest', 4, $config['largest'] ) ?> 
			<?php nktagcloud_form_select('Unit', 'unit', array('pt', 'px', '%', 'em', 'ex', 'mm') ); ?>
			<br />
			<?php nktagcloud_input( 'Numbers of tags to show', 'number', 4, $config['number'] ) ?> 
			<br />
			<?php nktagcloud_form_select('Format', 'format', array('flat', 'list') ); ?>
			<?php nktagcloud_form_select('Order', 'order', array('ASC', 'DESC', 'RAND' ) ); ?>
			<?php nktagcloud_form_select('Orderby', 'orderby', array('name', 'count', 'both' ) ); ?>
			<p>
				The 'both' option of <tt>Orderby</tt> will sort by post count first
				and then by name. It doesn't exist in the default tag cloud
				and will ignore the <tt>Order</tt> option.
			</p>
			
			<?php nktagcloud_form_select('Add post count to tags?', 'inject_count', array('No', 'Yes') ) ?>
			<?php nktagcloud_form_select('Put the post count outside of the hyperlink?', 'inject_count_outside', array('No', 'Yes') ) ?>
			<br />
			<?php nktagcloud_form_select('Add categories to tag cloud?', 'categories', array('No', 'Yes') ) ?>
			<br />
			<?php nktagcloud_form_select('Force tags with multiple words on one line?', 'replace', array('No', 'Yes') ) ?>
			<br />
			<?php nktagcloud_submit() ?>
			
			<h3>CSS settings</h2>
			<p>
				The list format is SEO-friendlier than the flat setting. By default this plugin includes CSS to make the list look like a normal cloud.  The CSS settings may or may not work on your template, depending on it's CSS. Please contact your theme developer for support if this doesn't work.
			</p>
			<?php nktagcloud_form_select('Remove CSS for list format that displays the tags inline?', 'rmcss', array('No', 'Yes') ) ?>
			<?php nktagcloud_form_select('Tag link decoration', 'decor', array('underline', 'none', 'don\'t touch' ) ) ?>
			<br />
			<?php nktagcloud_form_select('Remove link to widget website?', 'homelink', array('No', 'Yes') ); ?>
			<br />
			<?php nktagcloud_submit() ?>
			<h3>Exclude/Include tags</h2>
			<p>
				Comma separated list of tags (term_id) to exclude or include. For example, 'exclude=5,27' means that tags that have the term_id 5 or 27 will NOT be displayed. See <a href="http://codex.wordpress.org/Template_Tags/wp_tag_cloud">Template Tags/wp tag cloud</a>.
			</p>
			<?php nktagcloud_input( 'Exclude Tags', 'exclude', 40, $config['exclude'] ) ?> 
			<br />
			<?php nktagcloud_input( 'Include Tags', 'include', 40, $config['include'] ) ?> 
			<br />
			<?php nktagcloud_submit() ?>
		</form>

		<h3>Reset the tag cloud</h2>
		<p>
			Use this if you want to delete your configuration and restore the defaults.
		</p>
		<form action="" method="post" >
			<input type="hidden" name="reset" value="ihatephp" />
			<?php nktagcloud_submit( 'Reset the tag cloud', 'button-secondary' ) ?>
		</form>
	</div> <?php
}

/**
 * Prints a <select> and the listed <option>s
 * @param string $title Title for the dropdown
 * @param string $name Form input name
 * @param array $choice List of options
 * @param boolean $br print <br /> after the select
 */
function nktagcloud_form_select( $title, $name, $choice ) {
	print $title; ?>
	<select name="config[<?php echo $name; ?>]"><?php
		// FIXME $option or selected should be passed as a parameter
		$option = get_option( 'nktagcloud' );
		$select = $option['config'][$name];
		foreach ( $choice as $unit ) {
			if ( $unit == $select ) {
				echo "<option selected>$unit</option>\n";
			}
			else {
				echo "<option>$unit</option>\n";
			}
		} ?>
	</select> <?php
	if ($br) {
		echo '<br />';
	} ?>
	<?php
}

/**
 * Print an input field
 * @param string $title Title
 * @param string $name Input field name
 * @param integer $size Size
 * @param string $value Value
 * @param boolean $br Append <br />
 */
function nktagcloud_input( $title, $name, $size, $value ) { ?>
	<?php echo $title ?>
	<input name="config[<?php echo $name ?>]" type="text" size="<?php echo $size ?>" value="<?php echo $value ?>" /> <?php
	if ($br) echo '<br />';
}

/**
 * Print a submit button
 * @param string $value Value
 * @param string $class CSS class for the submit button
 */
function nktagcloud_submit( $value = 'Save changes', $class = 'button-primary' ) { ?>
	<input type="submit" class="<?php echo $class ?>" value="<?php echo $value ?>" /> <?php
}

?>
